
import argparse
from osiris.Osiris import Osiris
from osiris.CLI.EnvironmentCli import EnvironmentCli


def main():
    """
    Main function which allows to build the client for the Backup part of osiris.
    :param : (String) The command entered by the user.
    :return: Nothing.
    """
    # create the top-level parser
    parser = argparse.ArgumentParser(prog='Osiris', usage='Osiris <object> <action> <option>')
    parser.add_argument('object', help="Object on which the actions will be done. \n Exemple:System, backup, Osiris object: vm, flavor...")
    SystemCliParser = parser.add_subparsers(title="Actions that can be executed on the object")

    # create the parser for the "list" command
    OsirisList = SystemCliParser.add_parser('list', usage='osiris backup list [--all] : List all backups definition of the target system. --all list backups definition for all system working with the current plugin.')
    OsirisList.add_argument("--verbose", "-v", help="increase output verbosity", action="store_true")
    OsirisList.set_defaults(func=backupListCli)

    # create the parser for the "show" command
    OsirisShow = SystemCliParser.add_parser('show', usage='osiris backup show [--system <systemName>] <backupName> : Show instances of the backup definiton.')
    OsirisShow.add_argument("--verbose", "-v", help="increase output verbosity", action="store_true")
    OsirisShow.set_defaults(func=backupShowCli)

    # create the parser for the "define" command
    OsirisDefine = SystemCliParser.add_parser('define', usage='osiris backup define <backupName> : Define a new backup with the current selected instances.')
    OsirisDefine.add_argument("--verbose", "-v", help="increase output verbosity", action="store_true")
    OsirisDefine.set_defaults(func=backupDefineCli)

    # create the parser for the "export" command
    OsirisExport = SystemCliParser.add_parser('export', usage='osiris backup export <backupName> <path> : Export a backup definition to a path.')
    OsirisExport.add_argument("--verbose", "-v", help="increase output verbosity", action="store_true")
    OsirisExport.set_defaults(func=backupExportCli)

    # create the parser for the "import" command
    OsirisImport = SystemCliParser.add_parser('import', usage='osiris backup import <path> : Import a backup definition from a path.')
    OsirisImport.add_argument("--verbose", "-v", help="increase output verbosity", action="store_true")
    OsirisImport.set_defaults(func=BackupImportCli)

    # create the parser for the "run " command
    OsirisRun = SystemCliParser.add_parser('run ', usage='osiris backup run [--system <systemName>] <backupName> : Run a backup on the target system.')
    OsirisRun.add_argument("--verbose", "-v", help="increase output verbosity", action="store_true")
    OsirisRun.set_defaults(func=backupRunCli)

    # create the parser for the "delete" command
    OsirisDelete = SystemCliParser.add_parser('delete', usage='osiris backup delete [--system <systemName>] <backupName> : Delete a backup definition.')
    OsirisDelete.add_argument("--verbose", "-v", help="increase output verbosity", action="store_true")
    OsirisDelete.set_defaults(func=backupDeleteCli)

    args = parser.parse_args()
    args.func(args)


def backupListCli(args):
    """
    Calls the osiris function which allows to list all backups definition of the target system. --all list backups definition for all system working with the current plugin.

    :param args: List of the necessary arguments indicated by the user when the command is inputted.
    :return: Nothing.
    """
    verbose = args.verbose
    EnvironmentCli.verboseCli(verbose)
    osiris = Osiris(EnvironmentCli.getSystem(), EnvironmentCli.getPlugin())
    osiris.backupList()


def backupShowCli(args):
    """
    Calls the osiris function which allows to show instances of the backup definiton.

    :param args: List of the necessary arguments indicated by the user when the command is inputted.
    :return: Nothing.
    """
    verbose = args.verbose
    EnvironmentCli.verboseCli(verbose)
    osiris = Osiris(EnvironmentCli.getSystem(), EnvironmentCli.getPlugin())
    osiris.backupShow()


def backupDefineCli(args):
    """
    Calls the osiris function which allows to define a new backup with the current selected instances.

    :param args: List of the necessary arguments indicated by the user when the command is inputted.
    :return: Nothing
    """
    verbose = args.verbose
    EnvironmentCli.verboseCli(verbose)
    osiris = Osiris(EnvironmentCli.getSystem(), EnvironmentCli.getPlugin())
    osiris.backupDefine()


def backupExportCli(args):
    """
    Calls the osiris function which allows to export a backup definition to a path.

    :param args: List of the necessary arguments indicated by the user when the command is inputted.
    :return: Nothing
    """
    verbose = args.verbose
    EnvironmentCli.verboseCli(verbose)
    osiris = Osiris(EnvironmentCli.getSystem(), EnvironmentCli.getPlugin())
    osiris.backupExport()


def BackupImportCli(args):
    """
    Calls the osiris function which allows to import a backup definition from a path.

    :param args: List of the necessary arguments indicated by the user when the command is inputted.
    :return: Nothing
    """
    verbose = args.verbose
    EnvironmentCli.verboseCli(verbose)
    osiris = Osiris(EnvironmentCli.getSystem(), EnvironmentCli.getPlugin())
    osiris.BackupImport()


def backupRunCli(args):
    """
    Calls the osiris function which allows to run a backup on the target system.

    :param args: List of the necessary arguments indicated by the user when the command is inputted.
    :return: Nothing
    """
    verbose = args.verbose
    EnvironmentCli.verboseCli(verbose)
    osiris = Osiris(EnvironmentCli.getSystem(), EnvironmentCli.getPlugin())
    osiris.backupRun()


def backupDeleteCli(args):
    """
    Calls the osiris function which allows to delete a backup definition.

    :param args: List of the necessary arguments indicated by the user when the command is inputted.
    :return: Nothing
    """
    verbose = args.verbose
    EnvironmentCli.verboseCli(verbose)
    osiris = Osiris(EnvironmentCli.getSystem(), EnvironmentCli.getPlugin())
    osiris.backupDelete()


if __name__ == "__main__":
    main()



import argparse
from osiris.Osiris import Osiris
from osiris.CLI.EnvironmentCli import EnvironmentCli


def main():
    """
    Main function which allows to build the client for the object part of osiris. .

    :param : (String) The command entered by the user.
    :return: Nothing.
    """
    # create the top-level parser
    parser = argparse.ArgumentParser(prog='Osiris', usage='Osiris <object> <action> <option>')
    parser.add_argument('object', help="Object on which the actions will be done. \n Osiris object: vm, flavor...")
    ObjectSystemCli = parser.add_subparsers(title="Actions that can be executed on the object")

    # create the parser for the "introspect" command
    OsirisIntrospect = ObjectSystemCli.add_parser('introspect', usage='osiris <object> introspect : Introspect a specific object type on the target system')
    OsirisIntrospect.add_argument("--verbose", "-v", help="increase output verbosity", action="store_true")
    OsirisIntrospect.set_defaults(func=objectIntrospectCli)

    # create the parser for the "list" command
    OsirisList = ObjectSystemCli.add_parser('list', usage='osiris <object> list : List all object instances currently on the local database')
    OsirisList.add_argument("--verbose", "-v", help="increase output verbosity", action="store_true")
    OsirisList.set_defaults(func=objectListCli)

    # create the parser for the "create" command
    OsirisCreate = ObjectSystemCli.add_parser('create', usage='osiris <object> create [args...] : Create a new object instance (on the local database and on the target system)')
    OsirisCreate.add_argument("--verbose", "-v", help="increase output verbosity", action="store_true")
    OsirisCreate.set_defaults(func=objectCreateCli)

    # create the parser for the "select" command
    OsirisSelectAll = ObjectSystemCli.add_parser('select', usage='osiris <object> select [<id>... | --all] : Select all or specifics instances of the given object')
    OsirisSelectAll.add_argument("--verbose", "-v", help="increase output verbosity", action="store_true")
    OsirisSelectAll.add_argument('-id', action="store", nargs=-1, dest="id", type=str, help="instance id")
    OsirisSelectAll.add_argument('--all', "-a", action="store", nargs=1, dest="system", type=str, help="select all objects")
    OsirisSelectAll.set_defaults(func=objectSelectCli)

    # create the parser for the "unselect" command
    OsirisUnselectAll = ObjectSystemCli.add_parser('unselect', usage='osiris <object> unselect [<id>... | --all] : Unselect  all or specifics instances of the given object')
    OsirisUnselectAll.add_argument("--verbose", "-v", help="increase output verbosity", action="store_true")
    OsirisUnselectAll.set_defaults(func=objectUnselectCli)

    # create the parser for the "delete" command
    OsirisDelete = ObjectSystemCli.add_parser('delete', usage='osiris <object> delete <id> : Delete an instance of an object (on the local database and on the target system)')
    OsirisDelete.add_argument("--verbose", "-v", help="increase output verbosity", action="store_true")
    OsirisDelete.set_defaults(func=objectDeleteCli)

    args = parser.parse_args()
    args.func(args)


def objectIntrospectCli(args):
    """
    Calls the osiris function which allows to introspect a specific object type on the target system.
    At this time, introspect an object remove previous introspects.

    :param args: List of the necessary arguments indicated by the user when the command is inputted.
    :return: Nothing
    """
    objectName = args.object
    verbose = args.verbose
    EnvironmentCli.verboseCli(verbose)
    osiris = Osiris(EnvironmentCli.getSystem(), EnvironmentCli.getPlugin())
    osirirsSystemIntrospect = osiris.objectIntrospect(objectName)
    if osirirsSystemIntrospect >= 0:
        print("The " + objectName + " instropect has been successfully achieved.")
    else:
        print("The instropect hasn't come to an end.")


def objectListCli(args):
    """
    Calls the osiris function which allows to list all object instances currently on the local database.

    :param args: List of the necessary arguments indicated by the user when the command is inputted.
    :return: Nothing
    """
    verbose = args.verbose
    EnvironmentCli.verboseCli(verbose)
    osiris = Osiris(EnvironmentCli.getSystem(), EnvironmentCli.getPlugin())
    osiris.objectList(args.object)


def objectCreateCli(args):
    """
    Calls the osiris function which allows to create a new object instance (on the local database and on the target system).

    :param args: List of the necessary arguments indicated by the user when the command is inputted.
    :return: Nothing
    """
    verbose = args.verbose
    EnvironmentCli.verboseCli(verbose)
    osiris = Osiris(EnvironmentCli.getSystem(), EnvironmentCli.getPlugin())
    osiris.objectCreate(args.object)


def objectSelectCli(args):
    """
    Calls the osiris function which allows to select all or specifics instances of the given object.

    :param args: List of the necessary arguments indicated by the user when the command is inputted.
    :return: Nothing
    """
    verbose = args.verbose
    EnvironmentCli.verboseCli(verbose)
    osiris = Osiris(EnvironmentCli.getSystem(), EnvironmentCli.getPlugin())
    osiris.objectSelect(args.object)


def objectUnselectCli(args):
    """
    Calls the osiris function which allows to unselect all or specifics instances of the given object.

    :param args: List of the necessary arguments indicated by the user when the command is inputted.
    :return: Nothing
    """
    verbose = args.verbose
    EnvironmentCli.verboseCli(verbose)
    osiris = Osiris(EnvironmentCli.getSystem(), EnvironmentCli.getPlugin())
    osiris.objectUnselect(args.object)


def objectDeleteCli(args):
    """
    Calls the osiris function which allows to delete an instance of an object (on the local database and on the target system).

    :param args: List of the necessary arguments indicated by the user when the command is inputted.
    :return: Nothing
    """
    verbose = args.verbose
    EnvironmentCli.verboseCli(verbose)
    osiris = Osiris(EnvironmentCli.getSystem(), EnvironmentCli.getPlugin())
    objectName = args.object
    osiris.objectDelete(objectName, id)


if __name__ == "__main__":
    main()




import argparse
from osiris.Osiris import Osiris
from osiris.CLI.EnvironmentCli import EnvironmentCli
import os


def main():
    """
    Main function which allows to build the client for the system  part of osiris. .

    :param : (String) The command entered by the user.
    :return: Nothing.
    """
    # create the top-level parser
    parser = argparse.ArgumentParser(prog='Osiris', usage='Osiris <object> <action> <option>')
    parser.add_argument('object', help="System on which the actions will be done.")
    SystemCliParser = parser.add_subparsers(title="Actions that can be executed on the system")

    # create the parser for the "set" command
    OsirisSet = SystemCliParser.add_parser('set', usage='osiris system set [--plugin (-p) <pluginName>] [--system (-s) <systemName>] : Set target system')
    OsirisSet.add_argument("--verbose", "-v", help="increase output verbosity", action="store_true")
    OsirisSet.add_argument('--plugin', "-p", action="store", nargs=1, dest="plugin", type=str, help="<Plugin name>")
    OsirisSet.add_argument('--system', "-s", action="store", nargs=1, dest="system", type=str, help="<System name>")
    OsirisSet.set_defaults(func=setSysCli)

    # create the parser for the "get" command
    OsirisGet = SystemCliParser.add_parser('get', usage='osiris system get : Get taget system')
    OsirisGet.add_argument("--verbose", "-v", help="increase output verbosity", action="store_true")
    OsirisGet.set_defaults(func=getSysCli)

    # create the parser for the "add" command
    OsirisAdd = SystemCliParser.add_parser('add', usage='osiris system add pathToDefJson : Add a system with a definition json file.')
    OsirisAdd.add_argument('pathToDefJson', type=str, help="path definition system json file")
    OsirisAdd.add_argument("--verbose", "-v", help="increase output verbosity", action="store_true")
    OsirisAdd.set_defaults(func=systemAddCli)

    # create the parser for the "remove" command
    OsirisRemove = SystemCliParser.add_parser('remove', usage='osiris system remove [--plugin (-p) <pluginName>] [--system (-s) <systemName>] : Remove a system and all his data.')
    OsirisRemove.add_argument('--plugin', "-p", action="store", nargs=1, dest="plugin", type=str, help="<Plugin name>")
    OsirisRemove.add_argument('--system', "-s", action="store", nargs=1, dest="system", type=str, help="<System name>")
    OsirisRemove.add_argument("--verbose", "-v", help="increase output verbosity", action="store_true")
    OsirisRemove.set_defaults(func=systemRemoveCli)

    # create the parser for the "list" command
    OsirisList = SystemCliParser.add_parser('list', usage='osiris system list : List all systems known')
    OsirisList.add_argument("--verbose", "-v", help="increase output verbosity", action="store_true")
    OsirisList.set_defaults(func=systemListCli)

    # create the parser for the "introspect" command
    OsirisIntrospect = SystemCliParser.add_parser('introspect', usage='osiris system introspect : Introspect all objects on the target system')
    OsirisIntrospect.add_argument("--verbose", "-v", help="increase output verbosity", action="store_true")
    OsirisIntrospect.set_defaults(func=systemIntrospectCli)

    # create the parser for the "show" command
    OsirisShow = SystemCliParser.add_parser('show', usage='osiris system show : Show all objects instances currently on the local database')
    OsirisShow.add_argument("--verbose", "-v", help="increase output verbosity", action="store_true")
    OsirisShow.set_defaults(func=systemShowCli)

    # create the parser for the "selectAll" command
    OsirisSelectAll = SystemCliParser.add_parser('selectAll', usage='osiris system selectAll : Select all objects instances currently on the local database')
    OsirisSelectAll.add_argument("--verbose", "-v", help="increase output verbosity", action="store_true")
    OsirisSelectAll.set_defaults(func=systemSelectAllCli)

    # create the parser for the "unselectAll" command
    OsirisUnselectAll = SystemCliParser.add_parser('unselectAll', usage='osiris system unselectAll : Unselect all objects instances currentry on the local database')
    OsirisUnselectAll.add_argument("--verbose", "-v", help="increase output verbosity", action="store_true")
    OsirisUnselectAll.set_defaults(func=systemUnselectAllCli)

    args = parser.parse_args()
    args.func(args)


def setSysCli(args):
    """
    Assigns the system attribute according to the given parameters.

    :param args: List of the necessary arguments indicated by the user when the command is inputted.
    :return: Nothing
    """
    system = args.system[0]
    plugin = args.plugin[0]
    verbose = args.verbose
    EnvironmentCli.verboseCli(verbose)
    EnvironmentCli.setSystem(system)
    EnvironmentCli.setPlugin(plugin)


def getSysCli(args):
    """
    Displays  the system and plugin object of the current environment.

    :param args: List of the necessary arguments indicated by the user when the command is inputted.
    :return: Nothing
    """
    verbose = args.verbose
    EnvironmentCli.verboseCli(verbose)
    print("Here is the current system: \n" + "\nsystem: " + EnvironmentCli.getSystem() + "\nPlugin: " + EnvironmentCli.getPlugin())


def systemAddCli(args):
    """
    Calls the osiris function which allows to add a system to local ~/.osiris from the definition given in JSON file at path.

    :param args: List of the necessary arguments indicated by the user when the command is inputted.
    :return: Nothing
    """
    path = args.pathToDefJson
    verbose = args.verbose
    EnvironmentCli.verboseCli(verbose)
    EnvironmentCli.verboseCli(verbose)
    osirisSystemAdd = Osiris.systemAdd(path)
    if osirisSystemAdd == 0:
        print("The system was successfully added")
    elif osirisSystemAdd == -1:
        print("The specified file does not exist")
    elif osirisSystemAdd == -2:
        print("The file is incorrect (plugin or system missing)")
    elif osirisSystemAdd == -3:
        print("The name of the plugin or system is not written in the right format.(Osiris does not accept special characters. )")
    elif osirisSystemAdd == -4:
        print("there is an OS problem to read the file")
    elif osirisSystemAdd == -5:
        print("if decondig JSON failed")
    else:
        print("We encountered a problem loading the directory")


def systemRemoveCli(args):
    """
    Calls the osiris function which allows to remove locally a system definition.

    :param args: List of the necessary arguments indicated by the user when the command is inputted.
    :return: Nothing
    """
    verbose = args.verbose
    EnvironmentCli.verboseCli(verbose)
    osirirsSystemRemove = Osiris.systemRemove(EnvironmentCli.getSystem(), EnvironmentCli.getPlugin())
    if osirirsSystemRemove == 0:
        print("The system has been successfully removed")
    elif osirirsSystemRemove == -1:
        print("The name of the system or plugin is incorrect. (Osiris does not accept special characters)")
    elif osirirsSystemRemove == -2:
        print("The system was not found ")
    else:
        print("Unknown error")


def systemListCli(args):
    """
    Calls the osiris function which allows to list all systems known.

    :param args: List of the necessary arguments indicated by the user when the command is inputted.
    :return: Nothing
    """
    verbose = args.verbose
    EnvironmentCli.verboseCli(verbose)
    if not os.path.isdir(os.path.join(os.environ['HOME'], '.osiris', 'systems')):
        print("You have not initialized the system for this, please use the command 'osiris system add'.")
    else:
        print(Osiris.systemList())


def systemIntrospectCli(args):
    """
    Calls the osiris function which allows to introspect all objects on the target system.
    At this time, introspect a system remove previous introspects.

    :param args: List of the necessary arguments indicated by the user when the command is inputted.
    :return: Nothing
    """
    verbose = args.verbose
    EnvironmentCli.verboseCli(verbose)
    osiris = Osiris(EnvironmentCli.getSystem(), EnvironmentCli.getPlugin())
    osirirsSystemIntrospect = osiris.systemIntrospect()
    if osirirsSystemIntrospect >= 0:
        print("The instropect has been successfully achieved.")
    else:
        print("The instropect hasn't come to an end.")


def systemShowCli(args):
    """
    Calls the osiris function which allows to show all objects instances currently on the local database.

    :param args: List of the necessary arguments indicated by the user when the command is inputted.
    :return: Nothing
    """
    verbose = args.verbose
    EnvironmentCli.verboseCli(verbose)
    osiris = Osiris(EnvironmentCli.getSystem(), EnvironmentCli.getPlugin())
    print(osiris.systemShow())


def systemSelectAllCli(args):
    """
    Calls the osiris function which allows to select all objects instances currently on the local database.

    :param args: List of the necessary arguments indicated by the user when the command is inputted.
    :return: Nothing
    """
    verbose = args.verbose
    EnvironmentCli.verboseCli(verbose)
    osiris = Osiris(EnvironmentCli.getSystem(), EnvironmentCli.getPlugin())
    print(osiris.systemSelectAll())


def systemUnselectAllCli(args):
    """
    Calls the osiris function which allows to unselect all objects instances currentry on the local database.

    :param args: List of the necessary arguments indicated by the user when the command is inputted.
    :return: Nothing
    """
    verbose = args.verbose
    EnvironmentCli.verboseCli(verbose)
    osiris = Osiris(EnvironmentCli.getSystem(), EnvironmentCli.getPlugin())
    print(osiris.systemUnselectAll())


if __name__ == "__main__":
    main()



#!/usr/bin/env python3
# -*-coding:utf-8 -*

"""
This module contains the OsirisPlugin class.
"""

import os
import json
import re
import osiris.const as const
import logging


class OsirisPlugin:
    """
    The OsirisPlugin class contains all methods to permit the translation between Osiris and a (like Openstack)
    :attribute __name: String contains the name of the plugin.
    """

    __name = ""

    def __init__(self, name):
        """
        Constructs a new OsirisPlugin instance.

        :param name: String that contains the name of the plugin.
        :return: Returns nothing.
        """
        logging.info("Initialize Osiris plugin ...")
        self.__setName(name)
        checkingCode = self.__check()
        if(checkingCode != 0):
            print("Error while loading plugin ... Error code (see __check()): ", checkingCode)
            exit(-1)

    def __check(self):
        """
        Check if plugin is ok :
            * def.json file exists
            * plugin name match pattern
            * objects folder exists
            * there is at least one object type
            * object's names match pattern

        :return: (Integer)
            * 0 if plugin is ok
            * -1 if def.json file doesn't exist
            * -2 if plugin name doesn't match pattern
            * -3 if objects folder doesn't exists
            * -4 if there is 0 object type
            * -5 if object's names doesn't match pattern
        """
        pluginName = self.getName()
        pluginPath = os.path.join(self.getPath())

        # Test : def.json file exists
        if(not os.path.isfile(os.path.join(pluginPath, 'def.json'))):
            return -1

        # Test : plugin name match pattern
        pluginNamePattern = re.compile(const.PLUGINS_NAMES_PATTERN)
        if(not pluginNamePattern.match(pluginName)):
            return -2

        # Test : objects folder exists
        if(not os.path.isdir(os.path.join(pluginPath, "objects"))):
            return -3

        # Test : there is at least one object type
        objectsDef = self.getObjectsDef()
        if(len(objectsDef) <= 0):
            return -4

        # Test : object's names match pattern
        objectsNamesPattern = re.compile(const.OBJECTS_NAMES_PATTERN)
        for objectDef in objectsDef:
            if(not objectsNamesPattern.match(objectDef['name'])):
                return -5

        # OK!
        else:
            return 0

    def getName(self):
        """
        Getter for __name attribute.

        :return: Returns __name (String) attribute.
        """
        return self.__name

    def __setName(self, name):
        """
        Setter for __name attribute.

        :param name: String that contains the name of the plugin.
        :return: Returns nothing.
        """
        self.__name = name

    def getPath(self):
        """
        Path to rep plugin
        :return: Return path to rep plugin .
        """
        return os.path.join(os.path.dirname(os.path.realpath(__file__)), "plugins", self.getName())

    def getObjectsDef(self):
        """
        Get list of object definition

        :return: Returns a list with all objects definition of the current plugin.
        """

        dataList = []

        # Get the list of all object descriptor files
        path = os.path.join(self.getPath(), "objects/")
        allObjectsFiles = os.listdir(path)

        # Put on our list all the json data
        for i in allObjectsFiles:

            pluginObject = os.path.join(path, i)

            with open(pluginObject) as jsonData:
                dataList.append(json.load(jsonData))

        return dataList

    def getObjectDef(self, objectName):
        """
        Get objectName object definition.

        :param objectName: (String) Name of the object type.
        :return: (JSON) ObjectName object definition or empty json if objectName object doesn't exists.
        """
        # Get object descriptor files
        objpath = os.path.join(self.getPath(), "objects/", objectName + ".json")
        if os.path.isfile(objpath):
            with open(objpath) as jsonData:
                return json.load(jsonData)
        else:
            return {}



import argparse
from osiris.Osiris import Osiris
from osiris.CLI.EnvironmentCli import EnvironmentCli


def main():
    """
    Main function which allows to build the client for the Backup instance part of osiris. .

    :param : (String) The command entered by the user.
    :return: Nothing.
    """
    # create the top-level parser
    parser = argparse.ArgumentParser(prog='Osiris', usage='Osiris <object> <action> <option>')
    parser.add_argument('object', help="Backup on which the actions will be done.")
    parser.add_argument('execution', help="Backup on which the actions will be done.")
    SystemCliParser = parser.add_subparsers(title="Actions that can be executed on the instance object")

    # create the parser for the "list" command
    OsirisList = SystemCliParser.add_parser('list', usage='osiris backup execution list [--system <systemName>] [--backup <backupName>] [--all] : List executions of the current backup (or of all backups of the target system).')
    OsirisList.add_argument("--verbose", "-v", help="increase output verbosity", action="store_true")
    OsirisList.set_defaults(func=backupExecutionListCli)

    # create the parser for the "export" command
    OsirisExport = SystemCliParser.add_parser('export', usage='osiris backup execution export [--system <systemName>] <backupName> <timestamp> <path> : Export a backup execution to a path.')
    OsirisExport.add_argument("--verbose", "-v", help="increase output verbosity", action="store_true")
    OsirisExport .set_defaults(func=backupExecutionExportCli)

    # create the parser for the "import" command
    OsirisImport = SystemCliParser.add_parser('import', usage='osiris backup execution import <path> : Import a backup execution from a path.')
    OsirisImport.add_argument("--verbose", "-v", help="increase output verbosity", action="store_true")
    OsirisImport.set_defaults(func=backupExecutionImportCli)

    # create the parser for the "restore" command
    OsirisRestore = SystemCliParser.add_parser('restore', usage='osiris backup execution restore [--system <systemName>] <backupName> <timestamp> : Restore a backup execution into target system.')
    OsirisRestore.add_argument("--verbose", "-v", help="increase output verbosity", action="store_true")
    OsirisRestore.set_defaults(func=backupExecutionRestoreCli)

    # create the parser for the "delete" command
    OsirisDelete = SystemCliParser.add_parser('delete', usage='osiris backup execution delete [--system <systemName>] <backupName> <timestamp> : Delete a backup execution.')
    OsirisDelete.add_argument("--verbose", "-v", help="increase output verbosity", action="store_true")
    OsirisDelete.set_defaults(func=backupExecutionDeleteCli)

    args = parser.parse_args()
    args.func(args)


def backupExecutionListCli(args):
    """
    Calls the osiris function which allows to list executions of the current backup (or of all backups of the target system).

    :param args: List of the necessary arguments indicated by the user when the command is inputted.
    :return: Nothing
    """
    verbose = args.verbose
    EnvironmentCli.verboseCli(verbose)
    osiris = Osiris(EnvironmentCli.getSystem(), EnvironmentCli.getPlugin())
    osiris.backupExecutionList()


def backupExecutionExportCli(args):
    """
    Calls the osiris function which allows to export a backup execution to a path.

    :param args: List of the necessary arguments indicated by the user when the command is inputted.
    :return: Nothing
    """
    verbose = args.verbose
    EnvironmentCli.verboseCli(verbose)
    osiris = Osiris(EnvironmentCli.getSystem(), EnvironmentCli.getPlugin())
    osiris.backupExecutionExport()


def backupExecutionImportCli(args):
    """
    Calls the osiris function which allows to import a backup execution from a path.

    :param args: List of the necessary arguments indicated by the user when the command is inputted.
    :return: Nothing
    """
    verbose = args.verbose
    EnvironmentCli.verboseCli(verbose)
    osiris = Osiris(EnvironmentCli.getSystem(), EnvironmentCli.getPlugin())
    osiris.backupExecutionImport()


def backupExecutionRestoreCli(args):
    """
    Calls the osiris function which allows to restore a backup execution into target system.

    :param args: List of the necessary arguments indicated by the user when the command is inputted.
    :return: Nothing
    """
    verbose = args.verbose
    EnvironmentCli.verboseCli(verbose)
    osiris = Osiris(EnvironmentCli.getSystem(), EnvironmentCli.getPlugin())
    osiris.backupExecutionRestore()


def backupExecutionDeleteCli(args):
    """
    Calls the osiris function which allows to delete a backup execution.

    :param args: List of the necessary arguments indicated by the user when the command is inputted.
    :return: Nothing
    """
    verbose = args.verbose
    EnvironmentCli.verboseCli(verbose)
    osiris = Osiris(EnvironmentCli.getSystem(), EnvironmentCli.getPlugin())
    osiris.backupExecutionDelete()



# !/usr/bin/env python3
# -*-coding:utf-8 -*

import sqlite3
import os
import logging


class OsirisDB:

    """
    The OsirisDB class contains all the functions necessary to interact with the database.

    :attribute __dbFilePath: String that contains the path to DB file
    :attribute __dbConn: String - connection object
    :attribute __cursor: String - object represent a database cursor, which is used to manage the context of a fetch operation.
    """

    __dbFilePath = None
    __dbConn = None
    __cursor = None

    def __init__(self, system):
        """
        Constructs a new instance to connect a data base

        :param system: (OsirisSystem) System linked to the database.
        """
        logging.info("Initialize Osiris local database ...")
        self.__setDbFilePath(os.path.join(os.environ['HOME'], '.osiris', 'systems', system.getPlugin().getName(), system.getName(), 'osiris.db'))
        self.__loadDatabase()
        self.__open()

    def __getDbFilePath(self):
        """
        Getter for __dbFilePath attribute.

        :return: String that returns path to DB file.
        """
        return self.__dbFilePath

    def __setDbFilePath(self, path):
        """
        Setter for __dbFilePath attribute.

        :param path: String that contains the path to DB file.
        """
        self.__dbFilePath = path

    def __getDbConn(self):
        """
        Getter for __dbConn attribute.
        """
        return self.__dbConn

    def __setDbConn(self, dbConn):
        """
        Setter for __dbConn attribute.

        :param dbConn: the connection object
        """
        self.__dbConn = dbConn

    def __getCursor(self):
        """
        Getter for __cursor attribute.
        """
        return self.__cursor

    def __setCursor(self, cursor):
        """
        Setter for __cursor attribute.

        :param cursor:
        """
        self.__cursor = cursor

    def __loadDatabase(self):
        """
        Create a database connection to the SQLite database

        :return: Returns nothing
        """
        try:
            self.__setDbConn(sqlite3.connect(self.__getDbFilePath()))
        except sqlite3.Error as e:
            print('Error connecting to database ! >> Error : %s' % (' '.join(e.args)))
            exit(-1)

    def __formatColumnName(self, name):
        """
        Format a object attribute name in database column name.

        :param name: (String) Object's attribute's name
        :return: (String) Formated string
        """
        return name.replace(" ", "")

    def __open(self):
        """
        Connection to the SQLite database
        """
        self.__setCursor(self.__getDbConn().cursor())

    def __close(self):
        """
        Commit and Close the connection with the database
        """
        self.__getDbConn().commit()
        self.__getCursor().close()

    def createTable(self, objectName, objectAttributes):
        """
        Create table for the object if it doesn't exist

        :param objectName: the name of the object
        :param objectAttributes: object instances in json format
        """

        self.__open()

        req = "CREATE TABLE IF NOT EXISTS " + objectName + " (osiris_id INTEGER PRIMARY KEY AUTOINCREMENT, osiris_selected BOOLEAN DEFAULT false, "
        for attribute in objectAttributes:
            if(attribute['type'] == 'litteral'):
                req += self.__formatColumnName(attribute['name']) + " TEXT, "
            elif(attribute['type'] == 'reference'):
                req += self.__formatColumnName(attribute['name']) + " TEXT, "
            elif(attribute['type'] == 'data'):
                req += self.__formatColumnName(attribute['name']) + " TEXT, "
            else:
                raise Exception(objectName + '[' + attribute['name'] + "] Attribute must be litteral, reference or data ...")
        req = req[:-2] + ")"
        self.__getCursor().execute(req)
        self.__close()

    def insert(self, objectDef, data):
        """
        Insert the data in the database

        :param objectDef: object definition in json format
        :param data: object data
        """

        self.__open()

        req = "INSERT INTO " + objectDef["name"] + "("
        for attribute in objectDef["attributes"]:
            req += self.__formatColumnName(attribute['name']) + ", "
        req = req[:-2]
        req += ") VALUES \n"

        for element in data:
            req += '('
            for attribute in objectDef["attributes"]:
                if(attribute['type'] == "litteral"):
                    value = str(element[attribute['name']])
                    req += "null, " if not value else "'" + value.replace("'", "''") + "', "
                elif(attribute['type'] == "reference"):
                    value = str(element[attribute['name']])
                    req += "null, " if not value else "'" + value.replace("'", "''") + "', "
                elif(attribute['type'] == "data"):
                    value = str(element[attribute['name']])
                    req += "null, " if not value else "'" + value.replace("'", "''") + "', "
                else:
                    raise Exception("Attribute must be litteral, reference or data ...")
            req = req[:-2]
            req += "),\n"

        req = req[:-2]
        req += ';'
        self.__getCursor().execute(req)
        self.__close()

    def listInstances(self, objectName):
        """
        List all instances of object of type objectName on the local database.

        :param objectName: (String) Name of the object type to list.
        :return: (JSON) List of instances.
        """
        print("Not yet implemented")
        exit(-1)

    def flush(self, objectName):
        """
        Flush all data on local database for objectName objects.

        :param objectName: (String) Object type to flush.
        :return: 0 if ok, -1 else
        """
        self.__open()
        req = "DELETE FROM " + objectName + ";"
        self.__getCursor().execute(req)
        self.__close()

    def selectInstance(self, objectName, osirisID=None):
        """
        Select objectName instance with osirisID (or all instances) on local database.

        :param objectName: (String) Object type name to select.
        :param osirisID: (Integer) ID of object instance to select. If None, select all instances of objectName type.
        :return: (Integer) 0 if ok, -1 else
        """
        # Put "osiris_selected" field to 1 for osirisID object instance
        # (or all instances if osirisID is None) of table __formatColumnName(objectName)
        print("Not yet implemented")
        exit(-1)

    def unselectInstance(self, objectName, osirisID=None):
        """
        Unselect objectName instance with osirisID (or all instances) on local database.

        :param objectName: (String) Object type name to select.
        :param osirisID: (Integer) ID of object instance to unselect. If None, unselect all instances of objectName type.
        :return: (Integer) 0 if ok, -1 else
        """
        # Put "osiris_selected" field to 0 for osirisID object instance
        # (or all instances if osirisID is None) of table __formatColumnName(objectName)
        print("Not yet implemented")
        exit(-1)

    def deleteInstance(self, objectName, osirisID):
        """
        Delete instance of objectName type and osirisID from local database.

        :param objectName: (String)
        :param osirisID: (Integer)
        :return: (Integer) 0 if ok, -1 else
        """
        # Exec something like `DELETE FROM objectName WHERE osiris_id = osirisID`
        print("Not yet implemented")
        exit(-1)



#!/usr/bin/env python3
# -*-coding:utf-8 -*

"""
This module contains the Osiris class.
"""

from osiris.OsirisSystem import OsirisSystem
import os
import json
import subprocess
import stat
import shutil
import re
import osiris.const as const
import logging


class Osiris:

    """
    The Osiris class contains all Osiris actions (introspect, list, backup ...)

    :attribute __system: OsirisSystem instance associate to this Osiris Instance.
    """

    __system = None

    def __init__(self, systemName, pluginName):
        """
        Constructs a new Osiris instance and init environment variables

        :param systemName: String that contains the name of the system associate with this Osiris instance. By default, gets the value present in osiris_env.json.
        :param pluginName: String that contains the name of the plugin used by the system. By default, gets the value present in osiris_env.json
        :return: Returns nothing.
        """

        logging.info("Initialize Osiris ...")

        self.__setSystem(systemName, pluginName)

        # Init env var
        os.environ['OSIRIS_PLUGIN'] = self.__getSystem().getPlugin().getName()
        os.environ['OSIRIS_SYSNAME'] = self.__getSystem().getName()
        initScriptPath = os.path.join(self.__getSystem().getPlugin().getPath(), "scripts", "scriptInit")
        if(os.path.isfile(initScriptPath)):
            exec(open(initScriptPath).read())

    def __getSystem(self):
        """
        Getter for __system attribute.

        :return: Returns __system (OsirisSystem) attribute.
        """
        return self.__system

    def __setSystem(self, systemName, pluginName):
        """
        Setter for __system attribute.

        :param systemName: Name of the system associate with this Osiris instance. By default, gets the value present in osiris_env.json.
        :param pluginName: Name of the plugin used by the system. By default, gets the value present in osiris_env.json
        :return: Returns nothing.
        """
        self.__system = OsirisSystem(systemName, pluginName)

    def __substitute(self, cmd, params=None):
        """
        Replace @keys on cmd ny associated value in dict.

        :param cmd: (String[]) Command to complete
        :param params: (Dict) @keys : values to inject in cmd
        :return: (String[]) Command with values instead of @keys
        """

    def __execCmd(self, cmd):
        """
        Execute the command or the script passed on parameter

        :param cmd: String with the shell command or a script name present on the plugin script folder
        :return: Returns script or command output
        """
        potential_script_path = os.path.join(self.__getSystem().getPlugin().getPath(), "scripts", cmd[0])
        # if a script with cmd name exists
        if(os.path.isfile(potential_script_path)):

            logging.info("%s%s%s", "Giving execution right to script `", potential_script_path, "`")
            os.chmod(potential_script_path, stat.S_IRWXU)

            script = potential_script_path + ' ' + ' '.join(cmd[1:])
            logging.info("%s%s%s", "Executing script `", script, "`")
            data = json.loads(subprocess.check_output(script, shell=True, stderr=subprocess.DEVNULL).decode("utf-8"))

        # it's a command
        else:
            logging.info("%s%s%s", "Executing command `", ' '.join(cmd), "`")
            data = json.loads(subprocess.check_output(' '.join(cmd), shell=True, stderr=subprocess.DEVNULL).decode("utf-8"))

        return data

    @staticmethod
    def systemAdd(path):
        """
        Add a system to local ~/.osiris from the definition given in JSON file at path.

        :param path: (String) Path to the system definition JSON file.
        :return: (Integer) 0 if ok, -1 if file not found, -2 if file is incorrect (plugin or system missing), -3 if plugin or system doesn't match pattern (const.SYSTEMS_NAMES_PATTERN), -4 if there is an OS problem to read the file, -5 if decondig JSON failed
        """

        logging.info("Adding system ...")

        # Check if path point on a real file
        if os.path.isfile(path):

            # Open file and load JSON data
            try:
                with open(path, 'r') as f:
                    data = json.loads(f.read())
            except OSError:
                logging.error("%s%s%s", "`", path, "` cannot be opened. Please cheack path and access rights.")
                return -4
            except ValueError:
                logging.error("%s%s%s", "Decoding JSON of `", path, "` has failed. Please check typos.")
                return -5

            # Test if plugin and name values are present
            namesPattern = re.compile(const.SYSTEMS_NAMES_PATTERN)
            if('plugin' not in data or 'name' not in data):
                logging.error("%s%s%s", "Missing plugin or system name on `", path, "`")
                return -2

            # Test if plugin and name values match with pattern
            if(not namesPattern.match(data['plugin']) or not namesPattern.match(data['name'])):
                logging.error("%s%s%s", "Plugin or system name on `", path, "` doesn't match pattern ^[A-Za-z0-9_\\-.]+$")
                return -3

            # Create path `~/.osiris/systems/<pluginName>/<systemName>`
            pathToDefFolder = os.path.join(os.environ['HOME'], '.osiris', 'systems', data['plugin'], data['name'])
            os.makedirs(pathToDefFolder, exist_ok=True)

            # Insert json data into `~/.osiris/systems/<pluginName>/<systemName>/def.json`
            with open(os.path.join(pathToDefFolder, 'def.json'), 'w') as f:
                f.write(json.dumps(data, indent=4, sort_keys=True))

            # Return OK!
            logging.info("System added !")
            return 0

        else:
            logging.error("%s%s%s", "`", path, "` is not a file or cannot be opened. Please cheack path and access rights.")
            return -1

    @staticmethod
    def systemRemove(systemName, pluginName):
        """
        Remove locally a system definition.

        :param systemName: (String) Name of the system to delete.
        :param pluginName: (String) Name of the system's plugin.
        :return: (Integer) 0 if ok, -1 if incorrect systemName or pluginName, -2 if system not found
        """
        logging.info("Removing system ...")
        pluginNamePattern = re.compile(const.PLUGINS_NAMES_PATTERN)
        systemNamePattern = re.compile(const.SYSTEMS_NAMES_PATTERN)
        if(not pluginNamePattern.match(pluginName) or not systemNamePattern.match(systemName)):
            logging.error("Plugin or system name doesn't match pattern ^[A-Za-z0-9_\\-.]+$")
            return -1
        else:
            try:
                pathToDelete = os.path.join(os.environ['HOME'], '.osiris', 'systems', pluginName, systemName)
                shutil.rmtree(pathToDelete)
            except FileNotFoundError:
                logging.error("%s%s", "`", pathToDelete, "` is not found.")
                return -2

            logging.info("System removed !")
            return 0

    @staticmethod
    def systemList():
        """
        List all systems known.

        :return: Returns list of systems in JSON. Ex : {'pluginA': ['SystemA'], 'pluginB': ['SystemB1', 'SystemB2']}
        """
        sysList = {}
        path = os.path.join(os.environ['HOME'], '.osiris', 'systems')
        for i in os.listdir(path):
            sysList[i] = os.listdir(os.path.join(path, i))
        return sysList

    def systemIntrospect(self):
        """
        Introspect all objects on the target system.
        At this time, introspect a system remove previous introspects.

        :return: Returns number of objects introspected if ok, -1 else.
        """
        logging.info("Introspecting whole system ...")
        objectsDefs = self.__getSystem().getPlugin().getObjectsDef()
        totalNbObjects = 0
        for objectDef in objectsDefs:
            nbObjects = self.objectIntrospect(objectDef['name'])
            if(nbObjects == -1):
                logging.info("%s%s%s", "Error while introspecting `", objectDef['name'], "` objects ...")
                return -1
            else:
                totalNbObjects += nbObjects

        logging.info("Whole system has been introspected !")
        return totalNbObjects

    def systemShow(self):
        """
        Show all objects instances currently on the local database.

        :return: Returns all objects instances in JSON.
        """
        # Use objectList(self, objectName) (like systemIntrospect with objectIntrospect)
        print("Not yet implemented")
        exit(-1)

    def systemSelectAll(self):
        """
        Select all objects instances currently on the local database.

        :return: Returns number of objects selected if ok, -1 else.
        """
        # Use self.objectSelect(..)
        print("Not yet implemented")
        exit(-1)

    def systemUnselectAll(self):
        """
        Unselect all objects instances currentry on the local database.

        :return: Returns number of objects unselected if ok, -1 else.
        """
        # Use self.objectUnselect(..)
        print("Not yet implemented")
        exit(-1)

    def objectIntrospect(self, objectName):
        """
        Introspect a specific object type on the target system.
        At this time, introspect an object remove previous introspects.

        :param objectName: (String) Name of the object type to instropect.
        :return: Returns number of objects introspected if ok, -1 else.
        """

        logging.info("%s%s%s", "Introspecting `", objectName, "` ...")
        # get def object
        objectDef = self.__getSystem().getPlugin().getObjectDef(objectName)
        if(objectDef == {}):
            logging.error("%s%s%s%s%s", "`", objectName, "` is not found on `", self.__getSystem().getPlugin().getName(), "` plugin.")
            return -1

        # Create table for the object if it doesn't exist
        self.__getSystem().getDb().createTable(objectDef['name'], objectDef['attributes'])

        # Execute introspect command and get data from system
        introspectCommand = objectDef["actions"]["introspect"]
        data = self.__execCmd(introspectCommand)

        # Remove data present into database
        self.__getSystem().getDb().flush(objectDef['name'])

        # Insert data into database
        self.__getSystem().getDb().insert(objectDef, data)

        return len(data)

    def objectList(self, objectName):
        """
        List all object instances currently on the local database.

        :param objectName: (String) Name of the object type to list.
        :return: Returns list of object instances in JSON.
        """
        # Use `listInstances(self, objectName)` of `OsirisDB.py`
        print("Not yet implemented")
        exit(-1)

    def objectCreate(self, objectName, args=None):
        """
        Create a new object instance (on the local database and on the target system).

        :param objectName: (String) Name of the object type to create.
        :param args: (Dict) Arguments needed to create the instance (None by default).
        :return: Returns id of the new instance if ok, -1 else.
        """
        # Create object on local database
        # use OsirisPlugin : getObjectDef
        # use OsirisDB > insert(..)

        # Get command from plugin definition
        # reuse OsirisPlugin : getObjectDef

        # Subtitute variables of the command with args dict
        # use self __substitute(..)

        # Execute command to create object on system
        # use self __execCMD(..)

        print("Not yet implemented")
        exit(-1)

    def objectSelect(self, objectName, id=None):
        """
        Select all or specifics instances of the given object.

        :param objectName: (String) Name of the object type to select.
        :param id: (List of Strings) ID of the instance to select. If not provided (None by default), select all instances of this type.
        :return: Returns number of objects selected if ok, -1 else.
        """
        # Use `selectInstance(..)` of `OsirisDB`
        print("Not yet implemented")
        exit(-1)

    def objectUnselect(self, objectName, id=None):
        """
        Unselect all or specifics instances of the given object.

        :param objectName: (String) Name of the object type to unselect.
        :param id: (String) ID of the instance to unselect. If not provided (None by default), unselect all instances of this type.
        :return: Returns number of objects unselected if ok, -1 else.
        """
        # Use `unselectInstance(..)` of `OsirisDB`
        print("Not yet implemented")
        exit(-1)

    def objectDelete(self, objectName, id):
        """
        Delete an instance of an object (on the local database and on the target system).

        :param objectName: (String) Name of the object type to delete.
        :param id: (String) ID of the instance to delete.
        :return: Returns 0 if ok, -1 else.
        """
        # Use deleteInstance(..) from OsirisDB
        print("Not yet implemented")
        exit(-1)

    def backupList(self, all=False):
        """
        List all backups definition of the target system. --all list backups definition for all system working with the current plugin.

        :param all: (Boolean) If all is True, list all backups of all systems with target plugin. Else, by default, list all backups of target system.
        :return: Returns backups list in JSON.
        """
        print("Not yet implemented")
        exit(-1)

    def backupShow(self, backupName, systemName=None):
        """
        Show instances of the backup definiton.

        :param backupName: (String) Name of the backup to show.
        :param systemName: (String) Name oh the system of the backup. By default (None), target system.
        :return: Return description of the backup in JSON.
        """
        print("Not yet implemented")
        exit(-1)

    def backupDefine(self, backupName):
        """
        Define a new backup with the current selected instances.

        :param backupName: (String) Name of the new backup definition.
        :return: Returns 0 if ok, -1 else.
        """
        print("Not yet implemented")
        exit(-1)

    def backupExport(self, backupName, path):
        """
        Export a backup definition to a path.

        :param backupName: (String) Name of the backup to export.
        :param path: (String) Relative path where the backup will be export.
        :return: Returns 0 if ok, -1 else.
        """
        print("Not yet implemented")
        exit(-1)

    def backupImport(self, path):
        """
        Import a backup definition from a path.

        :param path: (String) Relative path where the backup to import is.
        :return: Returns 0 if ok, -1 else.
        """
        print("Not yet implemented")
        exit(-1)

    def backupRun(self, backupName, systemName=None):
        """
        Run a backup on the target system.

        :param backupName: (String) Name of the backup to run.
        :param systemName: (String) Name of the system where the backup come. By default (None), target system.
        :return: Returns number of instances backuped if ok, -1 else.
        """
        print("Not yet implemented")
        exit(-1)

    def backupDelete(self, backupName, systemName=None):
        """
        Delete a backup definition.

        :param backupName: (String) Name of the backup to delete.
        :param systemName: (String) Name of the system where the backup come. By default (None), target system.
        :return: Returns 0 if ok, -1 else.
        """
        print("Not yet implemented")
        exit(-1)

    def backupExecutionList(self, systemName=None, backupName=None, all=False):
        """
        List executions of the current backup (or of all backups of the target system).

        :param systemName: (String) Name of the system of backup executions. By default (None), target system.
        :param backupName: (String) Name of the backup definition of backup executions. By default (None), all backups of the system.
        :param all: (Boolean) If true, list backup executions of all systems of the target plugin. Else, just list backup executions of systemName (or target system by default).
        :return: Returns backup execution list in JSON.
        """
        print("Not yet implemented")
        exit(-1)

    def backupExecutionExport(self, backupName, timestamp, path, systemName=None):
        """
        Export a backup execution to a path.

        :param backupName: (String) Name of the backup of the backup execution.
        :param timestamp: (String) Timestamp (yyyymmddhhmmss) of the backup execution.
        :param path: (String) Relative path where the backup execution will be exported.
        :param systemName: (String) Name of the system of the backup execution. By default (None), target system.
        :return: Returns 0 if ok, -1 else.
        """
        print("Not yet implemented")
        exit(-1)

    def backupExecutionImport(self, path):
        """
        Import a backup execution from a path.

        :param path: (String) Relative path of the backup execution to import.
        :return: Returns 0 if ok, -1 else.
        """
        print("Not yet implemented")
        exit(-1)

    def backupExecutionRestore(self, backupName, timestamp, systemName=None):
        """
        Restore a backup execution into target system.

        :param backupName: (String) Name of the backup of the backup execution.
        :param timestamp: (String) Timestamp (yyyymmddhhmmss) of the backup execution.
        :param systemName: (String) Name of the system of backup execution. By default (None), target system.
        :return: Returns number of instances restored if ok, -1 else.
        """
        print("Not yet implemented")
        exit(-1)

    def backupExecutionDelete(self, backupName, timestamp, systemName=None):
        """
        Delete a backup execution.

        :param backupName: (String) Name of the backup of the backup execution.
        :param timestamp: (String) Timestamp (yyyymmddhhmmss) of the backup execution.
        :param systemName: (String) Name of the system of the backup execution. By default (None), target system.
        :return: Returns 0 if ok, -1 else.
        """
        print("Not yet implemented")
        exit(-1)



#!/usr/bin/env python

"""The setup script."""

from setuptools import setup, find_packages

with open('README.md') as readme_file:
    readme = readme_file.read()

with open('CHANGELOG.md') as changelog_file:
    changelog = changelog_file.read()

with open('AUTHORS.md') as authors_file:
    authors = authors_file.read()

requirements = []

setup(
    name='atos-osiris',
    version="0.0.5",
    author=authors,
    description="Perform generic actions on specific systems",
    long_description=readme + '\n\n' + changelog,
    long_description_content_type='text/markdown',
    url='https://git.atosone.com/osiris/osiris/',
    download='https://git.atosone.com/osiris/osiris/-/archive/master/osiris-master.zip',
    packages=find_packages(),
    python_requires='>=3.5',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Environment :: Console',
        'Framework :: Flake8',
        'Framework :: Pytest',
        'Framework :: Setuptools Plugin',
        'Framework :: tox',
        'Intended Audience :: System Administrators',
        'License :: Other/Proprietary License',
        'Natural Language :: English',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Topic :: System'
    ],
    license='Other/Proprietary License',
    entry_points={
        'console_scripts': [
            'osiris=osiris.CLI.OsirisCli:main'
        ]
    },
    package_data={
        "osiris": ["plugins/*/*.json", "plugins/*/*/*.json", "plugins/*/scripts/*"]
    },
    install_requires=requirements,
    include_package_data=True,
    keywords=['osiris', 'backup', 'restore', 'openstack'],
    platforms='any',
    zip_safe=True
)



import os
import json
import os.path
import logging


class EnvironmentCli:

    def checkout():
        """
        Checks if the .osiris folder and the file osiris_env.json exist and if not, creates it.

        :param : Nothing.
        :return: Nothing.
        """
        homeOsiris = os.environ['HOME']
        if not os.path.isdir(os.path.join(homeOsiris, ".osiris/")):
            os.mkdir(os.path.join(homeOsiris, ".osiris/"))
        if not os.path.isfile(os.path.join(homeOsiris, ".osiris/", "osiris_env.json")):
            env = {"OSIRIS_SYSTEM": "", "OSIRIS_PLUGIN": ""}
            with open(EnvironmentCli.getPath(), 'w') as f:
                json.dump(env, f)

    def getPath():
        """
        Gives the path where the environment definition should be placed in this specific case in the user's home.

        :param: It has no parameters
        :return: Return the path of the json file that allows to declare the environment.
        """
        # 'HOME' environment variable
        homeOsiris = os.environ['HOME']
        # Join various path components
        envJson = os.path.join(homeOsiris, ".osiris/", "osiris_env.json")
        return envJson

    def setSystem(system):
        """
        Assigns the system attribute according to the given parameters.

        :param system: Name of the system associate with this Osiris instance. By default, gets the value present in osiris_env.json.
        :return: Nothing.
        """
        with open(EnvironmentCli.getPath()) as f:
            env = json.load(f)
        env["OSIRIS_SYSTEM"] = system
        with open(EnvironmentCli.getPath(), 'w') as f:
            json.dump(env, f)

    def getSystem():
        """
        Returns the Osiris system object of the current environment.

        :param : Nothing.
        :return: Returns setSystem(system) attribute.
        """
        with open(EnvironmentCli.getPath()) as f:
            env = json.load(f)
            system = env["OSIRIS_SYSTEM"]
        return system

    def setPlugin(plugin):
        """
        Assigns the plugin attribute according to the given parameters.

        :param plugin: Name of the system associate with this Osiris instance. By default, gets the value present in osiris_env.json.
        :return: Nothing
        """
        with open(EnvironmentCli.getPath()) as f:
            env = json.load(f)
        env["OSIRIS_PLUGIN"] = plugin
        with open(EnvironmentCli.getPath(), 'w') as f:
            json.dump(env, f)

    def getPlugin():
        """
        Returns the Osiris plugin object of the current environment.

        :param : Nothing.
        :return: Returns setPlugin(plugin) attribute.
        """
        with open(EnvironmentCli.getPath()) as f:
            env = json.load(f)
            plugin = env["OSIRIS_PLUGIN"]
        return plugin

    def verboseCli(verbose):
        """
        Gives information on the actions that carry out the CLI in the background.

        :param args: List of the necessary arguments indicated by the user when the command is inputted.
        :return: :Nothing.
        """
        if verbose is True:
            logging.basicConfig(level=logging.INFO)



from osiris.CLI import SystemCli
from osiris.CLI import ObjectCli
from osiris.CLI import BackupCli
from osiris.CLI import BackupExecutionCli
from osiris.CLI.EnvironmentCli import EnvironmentCli
import sys
import os


def main():
    """
    Calls the osiris function which allows to unselect all or specifics instances of the given object.

    :param: Nothing.
    :return: Nothing.
    """
    args = sys.argv[1:]
    if args[0] == 'system':
        EnvironmentCli.checkout()
        SystemCli.main()

    elif args[0] == 'backup' and args[1] == 'execution':
        EnvironmentCli.checkout()
        BackupExecutionCli.main()

    elif args[0] == 'backup':
        EnvironmentCli.checkout()
        BackupCli.main()

    else:
        EnvironmentCli.checkout()
        if EnvironmentCli.getPlugin() == '':
            print("The plugin has not been specified")

        else:
            homeOsiris = os.path.join(os.path.dirname(os.path.realpath(__file__)), "../", "plugins", EnvironmentCli.getPlugin(), "objects")
            files = os.listdir(homeOsiris)
            find = False
            for name in files:
                if os.path.splitext(name)[0] == args[0]:
                    find = True
                    ObjectCli.main()
            if find is False:
                print("The object is not an object " + EnvironmentCli.getPlugin())


if __name__ == "__main__":
    main()



#!/usr/bin/env python3
# -*-coding:utf-8 -*

"""
This module contains the OsirisSystem class.
"""

from osiris.OsirisDB import OsirisDB
from osiris.OsirisPlugin import OsirisPlugin
import os
import logging


class OsirisSystem:

    """
    The OsirisSystem class contains all methods to interact with a concrete system.
    """

    __name = ""
    __plugin = None
    __db = None

    def __init__(self, systemName, pluginName):
        """
        Constructs a new OsirisSystem instance.

        :param systemName: String that contains the name of the system associate with this Osiris instance
        :param pluginName: String that contains the name of the plugin used by the system.
        :return: Returns nothing.
        """
        logging.info("Initialize Osiris system ...")
        self.__setPlugin(pluginName)
        self.__setName(systemName)
        self.__setDb(OsirisDB(self))

    def getName(self):
        """
        Getter for __name attribute.

        :return: Returns __name (String) attribute
        """
        return self.__name

    def __setName(self, name):
        """
        Setter for __name attribute.

        :param name: (String) System's name
        :return: Returns nothing.
        """
        path = os.path.join(self.getPath(), self.getPlugin().getName(), name, 'def.json')
        if os.path.isfile(path):
            self.__name = name
        else:
            print("no such file : ", path)
            exit(-1)

    def getPlugin(self):
        """
        Getter for __plugin attribute.

        :return: Returns __plugin (OsirisPlugin) attribute.
        """
        return self.__plugin

    def __setPlugin(self, pluginName):
        """
        Setter for __plugin attribut.

        :param name: (String) Plugin's name.
        :return: Returns nothing.
        """
        self.__plugin = OsirisPlugin(pluginName)

    def getDb(self):
        """
        Getter for __db attribute.

        :return: Returns __db (OsirisDB) attribute
        """
        return self.__db

    def __setDb(self, osirisDB):
        """
        Setter for __db attribute.

        :param osirisDB: OsirisDB object
        :return: Returns nothing
        """
        self.__db = osirisDB

    def getPath(self):
        """
        Path to rep system
        :return: Return path to rep system
        """
        return os.path.join(os.environ['HOME'], '.osiris', 'systems')
